# ShinyBibTeX v1.1
#
# R Shiny app for simple evaluations of bibtex files.
#
# © Cord Hockemeyer, 2024, University of Graz, AUSTRIA
# Email: cord.hockemeyer@uni-graz.at
# URL: https://homepage.uni-graz.at/en/hockemeyer-cord-dipl.-inf./
#
# License: GNU GPLv3 or newer

library(shiny)
library(rbibutils)
library(RefManageR)
library(ggplot2)

BibOptions(match.author = "exact")

staticbib <- as.BibEntry(readBib("empty.bib", direct = TRUE))
bib <- reactiveValues(val=staticbib)
contrbib <- reactiveValues(val=staticbib)
sry <- as.numeric(staticbib$year)
rawyear <- reactiveValues(val=sry)
year <- reactiveValues(val=sry[which(!is.na(sry))])

bt <- unique(staticbib$bibtype)
rfr <- sapply(bt, function(x) sum(staticbib$bibtype==x))
names(rfr) <- bt
rdf <- data.frame(Type=names(rfr), Count=rfr)
rdf$Percent <- rdf$Count / sum(rdf$Count) * 100
fr <- reactiveValues(val=rfr)
df <- reactiveValues(val=rdf)

al <- reactiveValues(val = rep(0,2))
freq <- reactiveValues(val = rep(0,2))

works <- reactiveValues(val=staticbib)
kwWorks <- reactiveValues(val=staticbib)
yearWorks <- reactiveValues(val=staticbib)


ui <- fluidPage(
  theme = "bootstrap.css",
  titlePanel(windowTitle = "ShinyBibTeX",
             title =
               div(
                 a(href="https://homepage.uni-graz.at/en/hockemeyer-cord-dipl.-inf./",
                   img(
                     src = "UniGraz.jpg",
                     height = 60, # originally 50
                     style = "margin:5px 5px"
                   )),
                 "ShinyBibTeX"
               )
  ),
  navbarPage("",
             tabPanel("Bib-File Selection",
                      fileInput("file",
                                "Choose BibTeX file",
                                accept = ".bib"),
                      includeHTML("fileRemarks.html"),
                      textOutput("loadMessage"),
                      HTML('<p>&nbsp;</p>')
             ),
             tabPanel("Annual Distribution",
                      plotOutput("annual")),
             tabPanel("Publication Types",
                      plotOutput("types"),
                      tableOutput("fr")),
             tabPanel("Top Contributors",
                      radioButtons("pubtype",
                                   label = "Type of publication",
                                   choiceNames = list("Journal article", "Book (Monograph)", "All"),
                                   choiceValues = list("article", "book", "all"),
                                   inline = TRUE),
                      radioButtons("contr",
                                   label = "Type of contribution",
                                   choiceNames = list("Author", "Editor"),
                                   choiceValues = list("author", "editor"),
                                   inline = TRUE),
                      HTML("<small>The aboved field is evaluated only if ALL publication types are selected.</small>"),
                      HTML("<p>&nbsp;</p>"),
                      numericInput("count", "Number of contributors to be shown", value = 15, min = 1, step = 1),
                      tableOutput("contributors")
             ),
             tabPanel("Author/Editor Search",
                      textInput("name", "Name to search for"),
                      HTML("<small>The name search does not (yet) work for non-ASCII characters; you can also search for parts of names</small><br>&nbsp;"),
                      radioButtons("what",
                                   label = "Type of contribution",
                                   choiceNames = list("Author", "Editor", "Author or Editor"),
                                   choiceValues = list("author", "editor", "both"),
                                   inline = TRUE),
                      textOutput("stats"),
                      HTML("<p>&nbsp;</p>"),
                      htmlOutput("selected")),
             tabPanel("Title Keyword Search",
                      textInput("keyword", "Keyword to search for"),
                      HTML("<small>This search ignores cases and applies regular expessions.</small>"),
                      HTML("<p>&nbsp;<hr></p>"),
                      textOutput("kwStats"),
                      HTML("<p>&nbsp;</p>"),
                      htmlOutput("kwSelected")
             ),
             tabPanel("Publication Year Search",
                      numericInput("year", "Publication year to search for", value = 2024),
                      # HTML("<small>This search ignores cases and applies regular expessions.</small>"),
                      HTML("<p>&nbsp;<hr></p>"),
                      textOutput("yearStats"),
                      HTML("<p>&nbsp;</p>"),
                      htmlOutput("yearSelected")
                      ),
             footer = 
               HTML('<p>&nbsp;<hr></p>ShinyBibTeX v. 1.1; &copy; 2024, <a href="https://homepage.uni-graz.at/en/hockemeyer-cord-dipl.-inf./" target="_blank">Cord Hockemeyer</a>, University of Graz, Austria <img src="gplv3-88x31.png" height=31 align="right" alt="Logo GNU GPLv3">')
  ),
)


# Define server logic required to draw a histogram
server <- function(input, output) {

  observeEvent(input$file, {
    bib$val <<- as.BibEntry(readBib(input$file$datapath, direct=TRUE, texChars = "convert"))
    output$loadMessage <- renderText(paste("Found", length(bib$val), "entries."))
    rawyear$val <<- as.numeric(bib$val$year)
    year$val <<- rawyear$val[which(!is.na(rawyear$val))]
    
    bt <<- unique((bib$val)$bibtype)
    fr$val <<- sapply(bt, function(x) sum((bib$val)$bibtype==x))
    names(fr$val) <<- bt
    df$val <<- data.frame(Type=names(fr$val), Count=fr$val)
    df$val$Percent <<- df$val$Count / sum(df$val$Count) * 100
    
    updateRadioButtons(inputId="what", selected="author")
    updateTextInput(inputId = "name", value="")
    
    works$val <- bib$val
    
    if (input$pubtype == "all")
      contrbib$val <<- bib$val
    else if (input$pubtype == "article")
      contrbib$val <<- SearchBib(bib$val, bibtype = "article")
    else if (input$pubtype == "book")
      contrbib$val <<- SearchBib(bib$val, bibtype = "book")
    
  })
  output$annual <- renderPlot(if (!is.null(bib$val)) hist(year$val, breaks = 50, main = "Publications over the years"))
  
  output$types <- renderPlot(if (!is.null(bib$val)) pie(fr$val, clockwise = TRUE, main = "Distribution of publication types"))
  output$fr <- renderTable(df$val, rownames=FALSE, colnames=TRUE)
  
  
  observeEvent(input$name, {
    updateRadioButtons(inputId="what", selected="author")
    if (is.null(input$name) || input$name=="") {
      works$val <- bib$val
    } else {
      a <- SearchBib(bib$val, author=input$name)
      e <- SearchBib(bib$val, editor=input$name)
      if (input$what == "author")
        works$val <- a
      else if (input$what == "editor")
        works$val <- e
      else if (input$what == "both") {
        works$val <- merge(a,e)
      }
      else
        works$val <- NULL
    }
  })
  observeEvent(input$what, {
    if (is.null(input$name) || input$name=="")
      works$val <- bib$val
    else {
      a <- SearchBib(bib$val, author=input$name)
      e <- SearchBib(bib$val, editor=input$name)
      if (input$what == "author")
        works$val <- a
      else if (input$what == "editor")
        works$val <- e
      else if (input$what == "both") {
        if (!inherits(a, "BibEntry")) {
          works$val <- e
        }
        else if (!inherits(e, "BibEntry")) {
          works$val <- a
        }
        else {
          works$val <- merge(a,e)
        }
      }
      else
        works$val <- NULL
    }
  })
  
  observeEvent(input$keyword, {
    if (is.null(input$keyword) || input$keyword == "")
      kwWorks$val <- bib$val
    else {
      kwWorks$val <- SearchBib(bib$val, title=input$keyword)
    }
  })
  
  observeEvent(input$year, {
    if (is.null(input$year) || input$year == "")
      yearWorks$val <- bib$val
    else {
      yearWorks$val <- SearchBib(bib$val, year=as.character(input$year))
    }
  })
  
  output$stats <- renderText({
    l <- length(works$val)
    if (is.null(input$name) || (input$name == ""))
      msg = ""
    else if (input$what == "author") {
      msg <- paste(l, "works were authored by", input$name)
    } else if (input$what == "editor") {
      msg <- paste(l, "works were edited by", input$name)
    } else
      msg <- paste(l, "works were authored and/or edited by", input$name)
    msg
  })

  output$kwStats <- renderText({
    l <- length(kwWorks$val)
    if (is.null(input$keyword) || (input$keyword==""))
      msg = ""
    else
      msg <- paste(l, "titles contain the keyword", input$keyword)
    msg
  })
  
  
  output$yearStats <- renderText({
    l <- length(yearWorks$val)
    msg <- paste(l, "publications were published in", input$year)
    msg
  })
  
  
    
  output$selected <- renderPrint({
    class(works$val) <- "bibentry"
    print(works$val, style="html")
    
  })
  
  output$kwSelected <- renderPrint({
    class(kwWorks$val) <- "bibentry"
    print(kwWorks$val, style="html")
  })
  
  output$yearSelected <- renderPrint({
    if (length(yearWorks$val) > 0) {
      class(yearWorks$val) <- "bibentry"
      print(yearWorks$val, style = "html")
    }
  })
  
    
  observeEvent(input$pubtype, {
    if (input$pubtype == "all")
      contrbib$val <<- bib$val
    else if (input$pubtype == "article")
      contrbib$val <<- SearchBib(bib$val, bibtype = "article")
    else if (input$pubtype == "book")
      contrbib$val <<- SearchBib(bib$val, bibtype = "book")
  })
  
  output$contributors <- renderTable({
    al <- sort(unlist(sapply(contrbib$val$author, function(x) {x$family})))
    el <- sort(unlist(sapply(contrbib$val$editor, function(x) {x$family})))
    if (input$contr == "author" || input$pubtype != "all")
      l <- al
    else if (input$contr == "editor")
      l <- el
    t <- data.frame(sort(table(l), decreasing = TRUE))
    names(t) <- c('Name', 'Freq.')
    len <- min(nrow(t), input$count)
    t[1:len,]
  },
  colnames = TRUE, rownames = FALSE)
}

# Run the application 
shinyApp(ui = ui, server = server)
